;;;
;;; ALTerator - ALT Linux configuration project
;;;
;;; Copyright (c) 2004,2005 ALT Linux Ltd.
;;; Copyright (c) 2004,2005 Alexey Voinov
;;; Copyright (c) 2004,2005 Stanislav Ievlev
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
;;; USA.
;;;
(use-modules (ice-9 regex) (srfi srfi-1) (srfi srfi-13))

(define *sandbox* #f)

(defmacro later x `(lambda () ,@x))

(define (sandbox)
  (if *sandbox* *sandbox*
      (begin (set! *sandbox* (make-module))
	     (module-define! *sandbox* 'lambda lambda)
	     (module-define! *sandbox* 'later later)
	     (module-define! *sandbox* 'make-regexp make-regexp)
	     (module-define! *sandbox* 'global global)
	     (module-define! *sandbox* 'quote quote)
	     *sandbox*)))

(define (cond-cons x y) (and x (cons x y)))

(define (load-mapper filename)
  (with-input-from-file filename
    (lambda () 
      (let loop ((e (read)))
	(if (eof-object? e) '()
	    (fold-right
	      (lambda (x y) (or (cond-cons (parse-mapper-item x) y) y))
	      (loop (read))
	      (perform-unquote e)))))))

(define (parse-mapper-item item)
  (and (pair? item)
       (cond-cons (convert-key (car item))
		  (convert-command (cdr item)))))

(define (convert-key key)
  (cond ((string? key) (lambda (x) (string=? key x)))
	((regexp? key) (lambda (x) (regexp-exec key x)))
	((null? key) (lambda (x) #t))
	((pair? key) (apply disjoin (map convert-key key)))
	((symbol? key) (convert-key (symbol->string key)))
	((procedure? key) key)
	(else #f)))

(define (convert-command cmd)
  (cond ((null? cmd)
	 '())
	((eq? 'file (car cmd))
	 (if (null? (cdr cmd)) '()
	     (cons* '/ctrl/look 'action 'file 'name (cadr cmd) (cddr cmd))))
	((eq? 'view (car cmd))
	 (if (null? (cdr cmd)) '()
	     (cons* '/ctrl/look 'action 'view 'id (cadr cmd) (cddr cmd))))
	(else
	  cmd)))

(define (map-view mapper id)
  (if (null? mapper) #f
      (or (test-and-eval ((caar mapper) id) (cdar mapper))
	  (map-view (cdr mapper) id))))

(define (test-and-eval ctxt answer)
  (if (not ctxt) #f
      (begin
	(if (vector? ctxt)
	    (module-define! (sandbox) 'match
			    (lambda args
			      (apply regexp-substitute #f ctxt args)))
	    (module-remove! (sandbox) 'match))
	(fold-right
	  (lambda (x y) (cons (if (procedure? x) (x) x) y)) '() answer))))

(define (perform-unquote e)
  (if (pair? e)
      (case (car e)
	((unquote) (list (eval (cadr e) (sandbox))))
	((unquote-splicing) (eval (cadr e) (sandbox)))
	((include) (fold (lambda (x y) (append y x)) '()
			 (map load-mapper (glob (cadr e)))))
	(else (list (fold-right append '() (map perform-unquote e)))))
      (list e)))

(define (disjoin . funcs)
  (lambda args (any? (lambda (f) (apply f args)) funcs)))

(define (any? pred lst)
  (if (null? lst) #f
      (or (pred (car lst))
	  (any? pred (cdr lst)))))
